SystemOrganization addCategory: #'Mondrian-FADELayout'!
SystemOrganization addCategory: #'Mondrian-FADELayout-Tests'!

MOLayout subclass: #MOFADELayout
	instanceVariableNames: 'numberOfIterations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-FADELayout'!

!MOFADELayout commentStamp: 'AlexandreBergel 2/21/2011 13:42' prior: 0!
FADE is a force directly algorithm (also called spring algorithm or spring layout) invented by Aaron Quigley (http://www.cs.st-andrews.ac.uk/~aquigley/?Research:Past_Projects:FADE). FADE is faster than MOForceBasedLayout when more than 350 edges and 350 nodes have to be layouted. !

MOFADELayout subclass: #MOFADE2Layout
	instanceVariableNames: 'count repellingForce attractingForce iterations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-FADELayout'!

!MOFADE2Layout methodsFor: 'accessing' stamp: 'EduardoFrias 1/25/2011 15:23'!
attractingForce: aBlock
	"Attracting force between edges."
	
	attractingForce := aBlock! !

!MOFADE2Layout methodsFor: 'private' stamp: 'EduardoFrias 1/25/2011 15:23'!
attractingForceFor: aFigure
	
	^attractingForce moValue: aFigure! !

!MOFADE2Layout methodsFor: 'accessing' stamp: 'EduardoFrias 1/25/2011 15:23'!
count 
	^ count! !

!MOFADE2Layout methodsFor: 'hook' stamp: 'EduardoFrias 1/26/2011 16:15'!
doExecute: graphElement
	| nodes nodeCenterA nodeCenterB deltaX deltaY distance xForces yForces xForce yForce force random edgesIndices nodeCenter nodeRadius nodeRadiusA nodeRadiusB nodeOrigin minX minY delta radial treeCode stack groupCenter groupRadius |
	"Start by placing the nodes at random"
	radial := MORadialTreeLayout new.
	radial fromPositions: self fromPositions.
	radial toPositions: self toPositions.
	radial executeOnNode: graphElement.
	self initializeConnectionPositions: graphElement.	"Init variables"
	random := Random new.	"Create array with the node center and radius. Like this we don't need to move the actual nodes"
	nodes := graphElement nodes asArray.
	nodeCenter := Array new: nodes size.
	nodeRadius := Array new: nodes size.
	treeCode := self treeCodeFor: graphElement.
	stack := OrderedCollection new.
	1 to: nodes size do: [ :index | 
		nodeCenter at: index put: (nodes at: index) center.
		nodeRadius at: index put: (nodes at: index) radius ].	"Create array with the indices of the edge nodes"
	edgesIndices := Array new: self edges size.
	1 to: self edges size do: [ :index | 
		| edge |
		edge := self edges at: index.
		edgesIndices at: index put: (Array with: (nodes indexOf: edge source) with: (nodes indexOf: edge target)) ].
	1 to: (self iterationsFor: graphElement) do: [ :iteration | 
		xForces := (Array new: nodes size) atAllPut: 0.
		yForces := (Array new: nodes size) atAllPut: 0.	"Node forces"
		1 to: nodes size do: [ :i |
			stack removeAll.
			stack add: treeCode.
			[stack size ~= 0] whileTrue: [
				| tree | 
				tree := stack removeLast.
				nodeCenterA := nodeCenter at: i.
				groupCenter := tree rectangle center.
				deltaX := groupCenter x - nodeCenterA x.
				deltaY := groupCenter y - nodeCenterA y.
				distance := (deltaX * deltaX + (deltaY * deltaY)) sqrt.
				"distance < 0.01
					ifTrue: [ 
						deltaX := random next + 5.
						deltaY := random next + 5.
						distance := (deltaX * deltaX + (deltaY * deltaY)) sqrt ].
				"
				(distance = 0) ifTrue: [ distance := 0.01 ].
				(tree rectangle width / distance <= self theta or: [ tree class name = #MONode])
					ifTrue: [ 
						nodeRadiusA := nodeRadius at: i.
						groupRadius := tree radius.
						force := (self repellingForceFor: graphElement) * nodeRadiusA * groupRadius / distance / distance.
						xForce := force * deltaX / distance.
						yForce := force * deltaY / distance.
						xForces at: i put: (xForces at: i) - xForce.
						yForces at: i put: (yForces at: i) - yForce. ] 
					ifFalse: [ 
						tree group do:[ :elm | stack add: elm ] ] ] ].
		edgesIndices
			do: [ :indices | 
				nodeCenterA := nodeCenter at: indices first.
				nodeCenterB := nodeCenter at: indices last.
				deltaX := nodeCenterB x - nodeCenterA x.
				deltaY := nodeCenterB y - nodeCenterA y.
				distance := (deltaX * deltaX + (deltaY * deltaY)) sqrt.
				distance < 0.01
					ifTrue: [ 
						deltaX := random next + 5.
						deltaY := random next + 5.
						distance := (deltaX * deltaX + (deltaY * deltaY)) sqrt ].
				distance := distance min: 15.
				force := (self attractingForceFor: graphElement) * distance.
				xForce := force * deltaX / distance.
				yForce := force * deltaY / distance.
				xForces at: indices first put: (xForces at: indices first) + xForce.
				yForces at: indices first put: (yForces at: indices first) + yForce.
				xForces at: indices last put: (xForces at: indices last) - xForce.
				yForces at: indices last put: (yForces at: indices last) - yForce ].	"Move centers"
		1 to: nodes size do: [ :index | 
			deltaX := xForces at: index.
			deltaY := yForces at: index.
			deltaX := deltaX min: 30.
			deltaX := deltaX max: -30.
			deltaY := deltaY min: 30.
			deltaY := deltaY max: -30.
			nodeCenter at: index put: ((nodeCenter at: index) translatedBy: deltaX @ deltaY) ] ].	"Node origins"
	nodeOrigin := Array new: nodes size.
	minX := 10000000.	"minX := Unlimited positive."
	minY := 10000000.	"minY := Unlimited positive."
	1 to: nodes size do: [ :index | 
		| origin node |
		node := nodes at: index.
		origin := (nodeCenter at: index) - ((node width / 2.0) @ (node height / 2.0)).
		nodeOrigin at: index put: origin.
		minX := minX min: origin x.
		minY := minY min: origin y ].	"Move nodes"
	minX := -1 * minX.
	minY := -1 * minY.
	delta := minX @ minY + (5 @ 5).
	1 to: nodes size do: [ :index | (nodes at: index) translateTo: (nodeOrigin at: index) + delta ]! !

!MOFADE2Layout methodsFor: 'initialization' stamp: 'EduardoFrias 1/25/2011 17:20'!
initialize
	
	super initialize.
	count := 0.
	iterations := 300.
	repellingForce := 100000000.
	attractingForce := 0.02! !

!MOFADE2Layout methodsFor: 'accessing' stamp: 'EduardoFrias 1/25/2011 15:23'!
iterations: aBlock
	
	iterations := aBlock! !

!MOFADE2Layout methodsFor: 'private' stamp: 'EduardoFrias 1/25/2011 15:23'!
iterationsFor: aFigure
	
	^iterations moValue: aFigure! !

!MOFADE2Layout methodsFor: 'accessing' stamp: 'EduardoFrias 1/25/2011 15:23'!
repellingForce: aBlock
	"Repelling force between nodes."
	
	repellingForce := aBlock! !

!MOFADE2Layout methodsFor: 'private' stamp: 'EduardoFrias 1/25/2011 15:23'!
repellingForceFor: aFigure
	
	^repellingForce moValue: aFigure! !

!MOFADELayout classSide methodsFor: 'as yet unclassified' stamp: 'EduardoFrias 12/29/2010 18:38'!
isAbstract 
	^ false! !

!MOFADELayout methodsFor: 'physic' stamp: 'EduardoFrias 1/25/2011 14:48'!
coulombRepulsion: node and: aTreeCode
	"mass of node times mass of other Nodes, divided by their squared distance"
	| d v totalForce rectangle tolerance |
	totalForce := 0.
	node attributeAt: #fadeCounter ifAbsentPut: [ 0 ].
	node attributeAt: #fadeCounter put: ((node attributeAt: #fadeCounter) + 1).

	rectangle := aTreeCode rectangle.
		
	v := node position - rectangle center.
	d := v r.
	d = 0.0 ifTrue: [ d := 0.01 ].
	tolerance := rectangle width / d.
	
	(tolerance <= self theta or: [ aTreeCode class name = #MONode ]) 
		ifTrue: 
			[ totalForce := totalForce + (v * 10000 * aTreeCode mass / (d * d * d)) ]
		ifFalse:
			[ aTreeCode group do: 
				[ :elm |
				totalForce := totalForce + (self coulombRepulsion: node and: elm)  
				] ].

	^ totalForce! !

!MOFADELayout methodsFor: 'configuration' stamp: 'EduardoFrias 1/12/2011 16:27'!
damping
	^ 0.5! !

!MOFADELayout methodsFor: 'hook' stamp: 'EduardoFrias 1/25/2011 14:52'!
doExecute: aGraphElement
	self numberOfIterations timesRepeat: 
		 [ self doIncrementalStep: aGraphElement ]


! !

!MOFADELayout methodsFor: 'hook' stamp: 'EduardoFrias 1/26/2011 18:18'!
doIncrementalStep: aGraphElement
	"Return the total energy "

	| stack totalKineticEnergy netForceX netForceY velocity nodesToLayout treeCode position |
	treeCode := self treeCodeFor: aGraphElement. 
	totalKineticEnergy := 0 @ 0.
	nodesToLayout := affectedNodes ifNil: [ aGraphElement nodes ] ifNotNil: [ affectedNodes ].
	stack := OrderedCollection new.
	
	nodesToLayout do: [ :node |
		stack removeAll.
		stack add: treeCode.
		"running sum of total force on this particular node"
		netForceX := 0.
		netForceY := 0.
		position := node position.
		[stack size ~= 0] 
			whileTrue: [
				| tree rectangle v d tolerance deltaX deltaY|
				tree := stack removeLast.
				rectangle := tree rectangle.
				
				deltaX := position x - rectangle center x.
				deltaY := position y - rectangle center y.
				
				d := ( deltaX * deltaX + (deltaY * deltaY) ) sqrt.
				"d = 0.0 ifTrue: [ d := 0.01 ]."
				d := d max: 50.
				tolerance := rectangle width / d.
	
				(tolerance <= self theta or: [ tree class name = #MONode ]) 
					ifTrue: 
						[ 
						| force forcex forcey |
						force := 10000 * tree mass / d / d.
						netForceX := netForceX + (force * deltaX / d) .
						netForceY := netForceY + (force * deltaY / d).
						]
					ifFalse:
						[
						tree group do: 
							[ :elm |
							stack add: elm.  
						] ].	
			].
		"netForce := self coulombRepulsion: node and: treeCode ."
		node incomingAndOutgoingEdgesDo: [ :edge |
			| f r ve vector force |
			"r := edge length.
			r = 0.0 ifTrue: [ ^ 1@1 ]".
			f := 30.
			vector := (edge target position - edge source position) .
			ve := vector  / f.
			force := (edge source == node)
						ifTrue: [ ve ]
						ifFalse: [ ve negated ].
			netForceX := netForceX + force x.
			netForceY := netForceY + force y ].
		
	"	velocity := ((node attributeAt: #velocity ifAbsentPut: [0]) +  netForce) * self damping."
		"node attributeAt: #velocity put: velocity."
"		(((node position + velocity) x > 0) and: [(node position + velocity) y > 0])
			ifTrue: [  (node isSelected and: [ (node hasAttribute: #fixed) not ]) ifFalse: [ node translateBy: velocity  ]]."
		
		netForceX := netForceX min: 30.
		netForceX := netForceX max: -30.
		netForceY := netForceY min: 30.
		netForceY := netForceY max: -30.
		node translateBy: (netForceX@netForceY) * self damping.
	].! !

!MOFADELayout methodsFor: 'hook' stamp: 'EduardoFrias 1/12/2011 16:30'!
doIncrementallyExecute: anElementNode
	"called by refreshLayoutEvery: ms. It is used to convey a feeling of incremental execution."
	
	self doIncrementalStep: anElementNode! !

!MOFADELayout methodsFor: 'hook' stamp: 'EduardoFrias 1/19/2011 19:13'!
doInitialize: aGraphElement
	| radial |
	
	radial := MORadialTreeLayout new.
	radial fromPositions: self fromPositions.
	radial toPositions: self toPositions.
	radial executeOnNode: aGraphElement.
	
	"| aRandom nodesToLayout numberOfNodes |
	aRandom := Random seed: 1.
	nodesToLayout := affectedNodes ifNil: [ aGraphElement nodes ] ifNotNil: [ affectedNodes ].
	numberOfNodes := nodesToLayout size.
	nodesToLayout do: [ :node | 
	"
		"make sure no 2 nodes are in exactly the same position"
		"node translateTo: (aRandom nextInt: numberOfNodes * 25 )@(aRandom nextInt: numberOfNodes * 25 ). ]."
	
	! !

!MOFADELayout methodsFor: 'physic' stamp: 'EduardoFrias 1/20/2011 14:27'!
hookeAttractionFor: node and: edge
	| f r v vector |
	r := edge length.
	r = 0.0 ifTrue: [ ^ 1@1 ].
	f := 30.
	vector := (edge target position - edge source position) / r.
	v := vector * r  / f.
	^ edge source == node
		ifTrue: [ v ]
		ifFalse: [ v negated ]
! !

!MOFADELayout methodsFor: 'initialize-release' stamp: 'EduardoFrias 1/24/2011 15:37'!
initialize
	super initialize.
	numberOfIterations := 200.! !

!MOFADELayout methodsFor: 'configuration' stamp: 'EduardoFrias 1/12/2011 16:28'!
lowThresholdEnergy
	^ 2! !

!MOFADELayout methodsFor: 'configuration' stamp: 'EduardoFrias 12/30/2010 17:11'!
minAndMaxPoints: aGraphElement
	| minX minY maxX maxY points |
	minX := Float infinity.
	minY := Float infinity.
	maxX := 0.
	maxY := 0.
	points := Array new: 2.
	
	aGraphElement nodes do: [ :node |
		minX > node position x ifTrue: [ minX := node position x ].
		minY > node position y ifTrue: [ minY := node position y ].
		maxX < node position x ifTrue: [ maxX := node position x ].
		maxY < node position y ifTrue: [ maxY := node position y ].
		].
	
	points at: 1 put: minX@minY.
	points at: 2 put: maxX@maxY.
	
	^points.
	! !

!MOFADELayout methodsFor: 'accessing' stamp: 'EduardoFrias 1/17/2011 15:15'!
numberOfIterations
	^ numberOfIterations! !

!MOFADELayout methodsFor: 'accessing' stamp: 'EduardoFrias 1/17/2011 15:14'!
numberOfIterations: aNumber
	numberOfIterations := aNumber.! !

!MOFADELayout methodsFor: 'configuration' stamp: 'EduardoFrias 1/5/2011 17:18'!
theta
	^ 1.0! !

!MOFADELayout methodsFor: 'configuration' stamp: 'EduardoFrias 1/24/2011 18:33'!
treeCode: groupOfNodes in: aRectangle
	| centerX centerY groups tree auxTree originX originY cornerX cornerY |
	groupOfNodes size = 1 ifTrue: [ ^groupOfNodes at: 1 ].
	groupOfNodes size = 0 ifTrue: [ ^nil ].
	
	groups := Array new: 4.
	tree := MOFadeGroup newIn: aRectangle.
	auxTree := MOFadeGroup newIn: aRectangle.
	centerX := aRectangle center x.
	centerY := aRectangle center y.
	originX := aRectangle origin x.
	originY := aRectangle origin y.
	cornerX := aRectangle corner x.
	cornerY := aRectangle corner y.	
	
	groups at: 1 put: OrderedCollection new.
	groups at: 2 put: OrderedCollection new.
	groups at: 3 put: OrderedCollection new.
	groups at: 4 put: OrderedCollection new.
	
	groupOfNodes do: [ :node |
		(node position x <= centerX and: [node position y <= centerY]) ifTrue: [ (groups at: 1) add: node ].
		(node position x > centerX and: [node position y <= centerY]) ifTrue: [ (groups at: 2) add: node ].
		(node position x <= centerX and: [node position y > centerY]) ifTrue: [ (groups at: 3) add: node ].
		(node position x > centerX and: [node position y > centerY]) ifTrue: [ (groups at: 4) add: node ]
		].

	auxTree add: (self treeCode: (groups at: 1) in: (Rectangle origin: originX@originY corner: centerX@centerY)).
	auxTree add: (self treeCode: (groups at: 2) in: (Rectangle origin: centerX@originY corner: cornerX@centerY)).
	auxTree add: (self treeCode: (groups at: 3) in: (Rectangle origin: originX@centerY corner: centerX@cornerY)).
	auxTree add: (self treeCode: (groups at: 4) in: (Rectangle origin: centerX@centerY corner: cornerX@cornerY)).
	
	(auxTree at: 1) ifNotNil: [ tree add: (auxTree at: 1) ].   
	(auxTree at: 2) ifNotNil: [ tree add: (auxTree at: 2) ].   
	(auxTree at: 3) ifNotNil: [ tree add: (auxTree at: 3) ].   
	(auxTree at: 4) ifNotNil: [ tree add: (auxTree at: 4) ].   
	
	^ tree! !

!MOFADELayout methodsFor: 'configuration' stamp: 'EduardoFrias 1/12/2011 18:55'!
treeCode: groupOfNodes in: aRectangle insideOf: anotherRectangle
	| centerX centerY groups tree auxTree originX originY cornerX cornerY |
	groupOfNodes size = 1 ifTrue: [ ^groupOfNodes at: 1 ].
	groupOfNodes size = 0 ifTrue: [ ^nil ].
	
	groups := Array new: 4.
	tree := MOFadeGroup newIn: aRectangle.
	
	aRectangle = anotherRectangle ifTrue: [
		groupOfNodes do: [ :node |
				tree add: node.
			].
		^ tree
		].
	
	auxTree := MOFadeGroup newIn: aRectangle.
	centerX := aRectangle center x.
	centerY := aRectangle center y.
	originX := aRectangle origin x.
	originY := aRectangle origin y.
	cornerX := aRectangle corner x.
	cornerY := aRectangle corner y.	
	
	groups at: 1 put: OrderedCollection new.
	groups at: 2 put: OrderedCollection new.
	groups at: 3 put: OrderedCollection new.
	groups at: 4 put: OrderedCollection new.
	
	groupOfNodes do: [ :node |
		(node position x <= centerX and: [node position y <= centerY]) ifTrue: [ (groups at: 1) add: node ].
		(node position x > centerX and: [node position y <= centerY]) ifTrue: [ (groups at: 2) add: node ].
		(node position x <= centerX and: [node position y > centerY]) ifTrue: [ (groups at: 3) add: node ].
		(node position x > centerX and: [node position y > centerY]) ifTrue: [ (groups at: 4) add: node ]
		].

	auxTree add: (self treeCode: (groups at: 1) in: (Rectangle origin: originX@originY corner: centerX@centerY) insideOf: aRectangle).
	auxTree add: (self treeCode: (groups at: 2) in: (Rectangle origin: centerX@originY corner: cornerX@centerY) insideOf: aRectangle).
	auxTree add: (self treeCode: (groups at: 3) in: (Rectangle origin: originX@centerY corner: centerX@cornerY) insideOf: aRectangle).
	auxTree add: (self treeCode: (groups at: 4) in: (Rectangle origin: centerX@centerY corner: cornerX@cornerY) insideOf: aRectangle).
	
	(auxTree at: 1) ~~ nil ifTrue: [ tree add: (auxTree at: 1) ].   
	(auxTree at: 2) ~~ nil ifTrue: [ tree add: (auxTree at: 2) ].   
	(auxTree at: 3) ~~ nil ifTrue: [ tree add: (auxTree at: 3) ].   
	(auxTree at: 4) ~~ nil ifTrue: [ tree add: (auxTree at: 4) ].   
	
	^ tree! !

!MOFADELayout methodsFor: 'configuration' stamp: 'EduardoFrias 1/12/2011 18:56'!
treeCodeFor: aGraphElement
	| tree groups nodes auxTree maxAndMinPoints rectangle centerX centerY originX originY cornerX cornerY |
	
	groups := Array new: 4.
	nodes := aGraphElement nodes.
	maxAndMinPoints := self minAndMaxPoints: aGraphElement.
	rectangle := Rectangle origin: (maxAndMinPoints at: 1)-(100@100) corner: (maxAndMinPoints at: 2)+(100@100).
	(rectangle origin x < 0 or: [rectangle origin y < 0]) ifTrue: [ rectangle origin: 0@0 ].
	auxTree := MOFadeGroup newIn: rectangle.
	tree := MOFadeGroup newIn: rectangle.
	centerX := rectangle center x.
	centerY := rectangle center y.
	originX := rectangle origin x.
	originY := rectangle origin y.
	cornerX := rectangle corner x.
	cornerY := rectangle corner y.	
	
	groups at: 1 put: OrderedCollection new.
	groups at: 2 put: OrderedCollection new.
	groups at: 3 put: OrderedCollection new.
	groups at: 4 put: OrderedCollection new.
	
	nodes do: [ :node |
		(node position x <= centerX and: [node position y <= centerY ]) ifTrue: [ (groups at: 1) add: node ].
		(node position x > centerX  and: [node position y <= centerY ]) ifTrue: [ (groups at: 2) add: node ].
		(node position x <= centerX  and: [node position y > centerY ]) ifTrue: [ (groups at: 3) add: node ].
		(node position x > centerX  and: [node position y > centerY ]) ifTrue: [ (groups at: 4) add: node ]
		].
	
	auxTree add: (self treeCode: (groups at: 1) in: (Rectangle origin: originX@originY corner: centerX@centerY) insideOf: rectangle).
	auxTree add: (self treeCode: (groups at: 2) in: (Rectangle origin: centerX@originY corner: cornerX@centerY) insideOf: rectangle).
	auxTree add: (self treeCode: (groups at: 3) in: (Rectangle origin: originX@centerY corner: centerX@cornerY) insideOf: rectangle).
	auxTree add: (self treeCode: (groups at: 4) in: (Rectangle origin: centerX@centerY corner: cornerX@cornerY) insideOf: rectangle).
	
	(auxTree at: 1) ~~ nil ifTrue: [ tree add: (auxTree at: 1) ].   
	(auxTree at: 2) ~~ nil ifTrue: [ tree add: (auxTree at: 2) ].   
	(auxTree at: 3) ~~ nil ifTrue: [ tree add: (auxTree at: 3) ].   
	(auxTree at: 4) ~~ nil ifTrue: [ tree add: (auxTree at: 4) ].   
	
	^tree! !

!MONode methodsFor: '*mondrian-fadelayout' stamp: 'EduardoFrias 1/24/2011 18:02'!
rectangle 
	^ Rectangle origin: (self position) corner: self position. ! !

TestCase subclass: #MOFADELayoutTest
	instanceVariableNames: 'view layout'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-FADELayout-Tests'!

!MOFADELayoutTest methodsFor: 'running' stamp: 'EduardoFrias 1/12/2011 17:28'!
setUp
	view := MOViewRenderer new.
	layout := MOFADELayout new.
	
	view interaction refreshLayoutEvery: 30.
	view shape rectangle size: 15.
	view nodes: (1 to: 5).
	view edges: (1 to: 5) from: #yourself to: [:v | v \\ 5].
	layout doInitialize: view root! !

!MOFADELayoutTest methodsFor: 'tests' stamp: 'EduardoFrias 1/17/2011 15:20'!
testBasic
	view := MOViewRenderer new.
	
	view shape rectangle withText.
	view nodes: Collection withAllSubclasses.
	view edgesFrom: #superclass.
	view layout: (MOFADELayout new numberOfIterations: 50).

	self shouldnt: [view root applyLayout] raise: Error. 
	view getRenderedForm.! !

!MOFADELayoutTest methodsFor: 'tests' stamp: 'EduardoFrias 1/12/2011 17:28'!
testCouloumbRepulsion
! !

!MOFADELayoutTest methodsFor: 'tests' stamp: 'EduardoFrias 1/19/2011 19:40'!
testIncomingAndOutgoingEdges

	| t |
	view := MOViewRenderer new.
	view nodes: (1 to: 5).
	view edges: {1 -> 2 . 1 -> 3} from: #key to: #value.
	
	t := 0.
	view root nodes first incomingAndOutgoingEdgesDo: [ :edge | t := t + 1 ].
	self assert: t = 2.

	t := 0.
	view root nodes second incomingAndOutgoingEdgesDo: [ :edge | t := t + 1 ].
	self assert: t = 1.

	t := 0.
	view root nodes third incomingAndOutgoingEdgesDo: [ :edge | t := t + 1 ].
	self assert: t = 1.
! !

!MOFADELayoutTest methodsFor: 'tests' stamp: 'AlexandreBergel 2/21/2011 13:23'!
testNodeSizeEffect
	"This test shows that the size of the node impacts its final location"
	
	| boundCenters view2 boundCenters2 | 
	view := MOViewRenderer new.
	view nodes: (1 to: 3).
	view edges: { 1 -> 2. 1 -> 3 . 2 -> 3} from: #key to: #value.
	view layout: (MOFADELayout new ).
	view root applyLayout.

	boundCenters := (view root nodes collect: #bounds) collect: #center.
	
	view2 := MOViewRenderer new. 
	view2 shape rectangle withText.
	view2 nodes: (1 to: 3).
	view edges: { 1 -> 2. 1 -> 3 . 2 -> 3} from: #key to: #value.
	view2 layout: (MOFADELayout new ).
	view2 root applyLayout.

	boundCenters2 := (view2 root nodes collect: #bounds) collect: #center.
	self assert: boundCenters2 asArray ~= boundCenters asArray! !

!MOFADELayoutTest methodsFor: 'tests' stamp: 'EduardoFrias 1/5/2011 17:55'!
testTreeCode
	| tree rectangle maxAndMinPoints comparingVisitor |
	
	maxAndMinPoints := layout minAndMaxPoints: view root.
	rectangle := Rectangle origin: (maxAndMinPoints at: 1)-(100@100) corner: (maxAndMinPoints at: 2)+(100@100).
	(rectangle origin x < 0 or: [rectangle origin y < 0]) ifTrue: [ rectangle origin: 0@0 ].
	
	tree := MOFadeGroup newIn: rectangle .
	tree add: (MOFadeGroup newIn: (Rectangle origin: rectangle origin corner: rectangle center)).
	tree add: (view root nodes at: 5).
	(tree at: 1) add: (MOFadeGroup newIn: (Rectangle origin: rectangle origin corner: rectangle center // 2)).
	(tree at: 1) add: (MOFadeGroup newIn: (Rectangle origin: (rectangle center x // 2@0) corner: rectangle center x@(rectangle center y // 2))).
	((tree at: 1) at: 1) add: (view root nodes at: 1).
	((tree at: 1) at: 1) add: (view root nodes at: 2).
	((tree at: 1) at: 2) add: (view root nodes at: 3).
	((tree at: 1) at: 2) add: (view root nodes at: 4). 
	
	comparingVisitor := FADEComparingVisitor new.
	comparingVisitor reference: (layout treeCodeFor: view root).
	
	self assert: (comparingVisitor isSame).
	tree acceptVisitor: comparingVisitor.
	self assert: (comparingVisitor isSame).
! !

TestCase subclass: #MOFadeGroupTest
	instanceVariableNames: 'fadeGroup'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-FADELayout-Tests'!

!MOFadeGroupTest methodsFor: 'running' stamp: 'EduardoFrias 1/5/2011 15:50'!
setUp
	fadeGroup := MOFadeGroup newIn: (Rectangle origin: 0@0 corner: 100@100).
	
	fadeGroup add: MONode new.
	fadeGroup add: (MOFadeGroup newIn: (Rectangle origin: 0@0 corner: 20@20)).
	(fadeGroup at: 2) add: MONode new.
	(fadeGroup at: 2) add: MONode new.! !

!MOFadeGroupTest methodsFor: 'tests' stamp: 'EduardoFrias 1/5/2011 15:50'!
testBasic
	
	self assert: fadeGroup mass = 3.
	self assert: (fadeGroup at: 2) mass = 2.
	self assert: (fadeGroup at: 1) class name = #MONode.
	self assert: ((fadeGroup at: 2) at: 1) class name = #MONode.
	self assert: ((fadeGroup at: 2) at: 2) class name = #MONode.
	self assert: fadeGroup rectangle center = (50@50)
	
	 
	
	! !

!MOFadeGroupTest methodsFor: 'tests' stamp: 'EduardoFrias 1/5/2011 14:36'!
testDistinctTrees
	| anotherFadeGroup comparingVisitor |
	anotherFadeGroup := MOFadeGroup newIn: 50@50.
	
	anotherFadeGroup add: (fadeGroup at: 1).
	anotherFadeGroup add: (MOFadeGroup newIn: 10@10).
	(anotherFadeGroup at: 2) add: ((fadeGroup at: 2) at: 1).
	
	comparingVisitor := FADEComparingVisitor new.
	comparingVisitor reference: (fadeGroup).
	
	self assert: (comparingVisitor isSame).
	anotherFadeGroup acceptVisitor: comparingVisitor.
	self deny: (comparingVisitor isSame).
	! !

!MOFadeGroupTest methodsFor: 'tests' stamp: 'EduardoFrias 1/4/2011 17:54'!
testVisitor
	
	| visitor |
	visitor := FADECountingVisitor new.
	
	self assert: (visitor numberOfNodes = 0).
	self assert: (visitor numberOfLeaves = 0).
	
	fadeGroup acceptVisitor: visitor.
	
	self assert: (visitor numberOfNodes = 5).
	self assert: (visitor numberOfLeaves = 3).! !

Object subclass: #FADEVisitor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-FADELayout'!

FADEVisitor subclass: #FADEComparingVisitor
	instanceVariableNames: 'isSame reference'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-FADELayout'!

!FADEComparingVisitor methodsFor: 'initialize-release' stamp: 'EduardoFrias 1/4/2011 18:13'!
initialize 
	super initialize.
	isSame := true	! !

!FADEComparingVisitor methodsFor: 'accessing' stamp: 'EduardoFrias 1/4/2011 18:12'!
isSame
	^ isSame ! !

!FADEComparingVisitor methodsFor: 'accessing' stamp: 'EduardoFrias 1/4/2011 18:10'!
reference
	^ reference! !

!FADEComparingVisitor methodsFor: 'accessing' stamp: 'EduardoFrias 1/4/2011 18:10'!
reference: anObject
	reference := anObject! !

!FADEComparingVisitor methodsFor: 'accessing' stamp: 'EduardoFrias 1/5/2011 15:14'!
setAsDifferent

	isSame := false! !

!FADEComparingVisitor methodsFor: 'visiting' stamp: 'EduardoFrias 1/5/2011 15:14'!
visitGroup: aGroup
	| i |
	(reference class name ~= #MOFadeGroup or: [aGroup group size ~= reference group size])
		ifTrue: [ self setAsDifferent ].
	
	i := 1.
	
	aGroup group do: [ :elm | 
		| previousReference |
		previousReference := reference.
		reference class name = #MOFadeGroup 
			ifTrue: 
				[ reference := reference group at: i ].
		elm acceptVisitor: self.
		reference := previousReference.
		i := i + 1.
		 ]! !

!FADEComparingVisitor methodsFor: 'visiting' stamp: 'EduardoFrias 1/4/2011 18:30'!
visitNode: aNode
	(reference = aNode) ifFalse: [ isSame := false ]! !

FADEVisitor subclass: #FADECountingVisitor
	instanceVariableNames: 'numberOfLeaves numberOfNodes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-FADELayout'!

!FADECountingVisitor methodsFor: 'initialize-release' stamp: 'EduardoFrias 1/4/2011 18:03'!
initialize 
	super initialize.
	numberOfLeaves := 0.
	numberOfNodes := 0.! !

!FADECountingVisitor methodsFor: 'accessing' stamp: 'EduardoFrias 1/4/2011 18:02'!
numberOfLeaves 
	^ numberOfLeaves ! !

!FADECountingVisitor methodsFor: 'accessing' stamp: 'EduardoFrias 1/4/2011 18:02'!
numberOfNodes 
	^ numberOfNodes ! !

!FADECountingVisitor methodsFor: 'visit' stamp: 'EduardoFrias 1/4/2011 18:14'!
visitGroup: aGroup
	super visitGroup: aGroup.
	numberOfNodes := numberOfNodes + 1.
	
	! !

!FADECountingVisitor methodsFor: 'visit' stamp: 'EduardoFrias 1/4/2011 18:07'!
visitNode: aObject
	numberOfLeaves := numberOfLeaves + 1.
	numberOfNodes := numberOfNodes + 1.! !

!FADEVisitor methodsFor: 'visiting' stamp: 'EduardoFrias 1/4/2011 18:14'!
visitGroup: aGroup

	aGroup group do: [ :elm | elm acceptVisitor: self ]! !

!FADEVisitor methodsFor: 'visiting' stamp: 'EduardoFrias 1/4/2011 18:14'!
visitNode: aNode
! !

Object subclass: #MOFADEForceBasedBenchmarks
	instanceVariableNames: 'fadeMedition forcebasedMedition fadeTime forceBasedTime'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-FADELayout-Tests'!

!MOFADEForceBasedBenchmarks methodsFor: 'accessing' stamp: 'AlexandreBergel 5/18/2011 19:34'!
calculateTimesUntil: aNumberOfNodes
	| nodes |
	nodes := 100.
	[ nodes <= aNumberOfNodes ] whileTrue: [
		self timeBenchmarkFor: 15 and: nodes.
		fadeMedition add: fadeTime.
		forcebasedMedition add: forceBasedTime.
		nodes := nodes + 100.
		] ! !

!MOFADEForceBasedBenchmarks methodsFor: 'initialization' stamp: 'AlexandreBergel 5/18/2011 19:34'!
initialize 
	super initialize.
	fadeTime := 0.
	forceBasedTime := 0.
	forcebasedMedition := OrderedCollection new.
	fadeMedition := OrderedCollection new.! !

!MOFADEForceBasedBenchmarks methodsFor: 'accessing' stamp: 'AlexandreBergel 5/18/2011 19:35'!
timeBenchmarkFor: numberOfIterations and: numberOfNodes
	| i view fadeLayout forceBasedLayout |
	i := 1.
	fadeLayout := MOFADELayout new.
	forceBasedLayout := MOForceBasedLayout new.
	view := MOViewRenderer new.
	view shape rectangle withText.
	view nodes: (Object withAllSubclasses copyFrom: 1 to: numberOfNodes).
	view edgesFrom: #superclass.
	view layout: (MOFADELayout new).
	
	fadeLayout edges: view root edges.
	fadeLayout numberOfIterations: numberOfIterations.
	fadeTime := [fadeLayout doExecute: view root] timeToRun. 
	
	forceBasedLayout edges: view root edges.
	forceBasedLayout iterations: numberOfIterations.
	forceBasedTime := [forceBasedLayout doExecute: view root] timeToRun! !

Object subclass: #MOFADESpringBenchmarks
	instanceVariableNames: 'fadeMedition springMedition fadeTime springTime'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-FADELayout-Tests'!

!MOFADESpringBenchmarks methodsFor: 'benchmarks' stamp: 'AlexandreBergel 1/24/2011 18:44'!
benchmack
	"
	self new benchmack
	
	"

	^ [| view |
		view := MOViewRenderer new.
		view shape rectangle withText.
		view nodes: Collection withAllSubclasses.
		view edgesFrom: #superclass.
		view layout: (MOFADELayout new).
		view root applyLayout] timeToRun.   ! !

!MOFADESpringBenchmarks methodsFor: 'benchmarks' stamp: 'AlexandreBergel 1/24/2011 18:52'!
benchmack2
	"
	self new benchmack2
	
	"

	^ [| view |
		view := MOViewRenderer new.
		view shape rectangle withText.
		view nodes: Collection withAllSubclasses.
		view edgesFrom: #superclass.
		view layout: (MOForceBasedLayout new).
		view root applyLayout] timeToRun.   ! !

!MOFADESpringBenchmarks methodsFor: 'accessing' stamp: 'AlexandreBergel 5/18/2011 19:34'!
calculateTimesUntil: aNumberOfNodes
	| nodes |
	nodes := 100.
	[ nodes <= aNumberOfNodes ] whileTrue: [
		self timeBenchmarkFor: 100 and: nodes.
		fadeMedition add: fadeTime.
		springMedition add: springTime.
		nodes := nodes + 100.
		] ! !

!MOFADESpringBenchmarks methodsFor: 'initialization' stamp: 'AlexandreBergel 5/18/2011 19:34'!
initialize
	super initialize.
	fadeTime := 0.
	springTime := 0.
	springMedition := OrderedCollection new.
	fadeMedition := OrderedCollection new.! !

!MOFADESpringBenchmarks methodsFor: 'accessing' stamp: 'AlexandreBergel 5/18/2011 19:35'!
timeBenchmarkFor: numberOfIterations and: numberOfNodes
	| i view fadeLayout springLayout |
	i := 1.
	fadeLayout := MOFADELayout new.
	springLayout := MOSpringLayout new. 
	view := MOViewRenderer new.
	view interaction refreshLayoutEvery: 30.
	view shape rectangle size: 15.
	view nodes: (1 to: numberOfNodes).
	view edges: (1 to: numberOfNodes) from: #yourself to: [:v | v \\ 5].
	fadeTime := [[ i <= numberOfIterations ] whileTrue:
		[ 
		fadeLayout doIncrementalStep: view root.
		i := i + 1.]] timeToRun.
	springLayout doInitialize: view root.
	i := 1.
	springTime := [[ i <= numberOfIterations ] whileTrue:
		[
		springLayout doIncrementalStep: view root.
		i := i + 1.]] timeToRun! !

Object subclass: #MOFadeGroup
	instanceVariableNames: 'group rectangle totalMass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-FADELayout'!

!MOFadeGroup classSide methodsFor: 'instance creation' stamp: 'EduardoFrias 1/5/2011 15:49'!
newIn: aRectangle
	| center instance |
	instance := self new.
	instance rectangle: aRectangle.
	^ instance! !

!MOFadeGroup methodsFor: 'visit' stamp: 'EduardoFrias 1/4/2011 17:57'!
acceptVisitor: visitor
	visitor visitGroup: self.! !

!MOFadeGroup methodsFor: 'adding' stamp: 'EduardoFrias 1/4/2011 15:53'!
add: aObject
	group add: aObject.
	! !

!MOFadeGroup methodsFor: 'accessing' stamp: 'EduardoFrias 1/4/2011 15:17'!
at: aNumber
	^ group at: aNumber.! !

!MOFadeGroup methodsFor: 'accessing' stamp: 'EduardoFrias 1/4/2011 15:19'!
group 
	^ group! !

!MOFadeGroup methodsFor: 'initialize-release' stamp: 'EduardoFrias 1/5/2011 15:47'!
initialize 
	super initialize.
	group := OrderedCollection new.
	rectangle := Rectangle origin: 0@0 corner: 0@0.! !

!MOFadeGroup methodsFor: 'accessing-computing' stamp: 'EduardoFrias 1/24/2011 18:30'!
mass
	totalMass ifNotNil: [ ^ totalMass ].
	totalMass := 0.
	self group do: [ :elm | totalMass := totalMass + elm mass ].
	^ totalMass! !

!MOFadeGroup methodsFor: 'accessing-computing' stamp: 'EduardoFrias 1/25/2011 16:20'!
radius
	^ (rectangle origin dist: rectangle corner) / 2 ! !

!MOFadeGroup methodsFor: 'accessing' stamp: 'EduardoFrias 1/5/2011 15:46'!
rectangle
	^ rectangle.! !

!MOFadeGroup methodsFor: 'accessing' stamp: 'EduardoFrias 1/5/2011 15:46'!
rectangle: aRectangle
	rectangle := aRectangle.! !

!Object methodsFor: '*mondrian-fadelayout' stamp: 'EduardoFrias 1/4/2011 18:07'!
acceptVisitor: visitor
	visitor visitNode: self.! !

!Object methodsFor: '*mondrian-fadelayout' stamp: 'EduardoFrias 1/4/2011 16:06'!
mass
	^ 1.! !
